package fxKohtaushallinta;

import java.util.ArrayList;
import java.util.List;

public class Avainsanat {
    private String[] sanat;
    private int sanoja;
    
  
    /**
     * Lukee 
     * @param koko Avainsanataulukon koko
     * @param sanat avainsanat välilyönnillä eroteltuina
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat(5, "   Yksi Kaksi Kolme    Neljä Viisi    Kuusi");
     *  a.montaSanaa() === 5;
     *  a.haeIndeksista(1) === "Kaksi";
     *  a.haeIndeksista(4) === "Viisi";
     *  a = new Avainsanat(6, "Yksi Kaksi Kolme Neljä");
     *  a.montaSanaa() === 4;
     * </pre>
     */
    public Avainsanat(int koko, String sanat) {
        this.sanat = new String[koko];
        this.sanoja = 0;
        String[] uusi = sanat.split("\\s+");
        for (String s : uusi) {
            if (s.length() > 0 ) {
                this.sanat[this.sanoja++] = s;
                if (this.sanoja == this.sanat.length) break;
            }
            
        }
    }
    
    
    public int montaSanaa() {
        return sanoja;        
    }
    
    
    public int getKoko() {
        return this.sanat.length;
    }
    
    
    public String get(int indeksi) {
        if (indeksi > -1 && indeksi < this.sanoja) return this.sanat[indeksi];
        return "";
    }
    
    
    /**
     * Lisää listaan avainsanat, joita siinä ei jo ole
     * @param lista lista, johon lisätään
     * @example
     * <pre name="test">
     * 
     * </pre>
     */
    public void lisaaSanatListaan(List<String> lista) {
        for (String sana : this.sanat)
            if (!lista.contains(sana)) lista.add(sana);        
    }
    
    
    /**
     * Lisää sanan, jos on tilaa
     * @param sana lisättävä sana
     */
    public void lisaaSana(String sana) {
        if (this.sanoja < this.sanat.length && haeIndeksi(sana) == -1)
            this.sanat[this.sanoja++] = sana;
    }
    
    
    /**
     * Jos sana löytyy taulukosta, poistaa sen. Siirtää taulukon viimeisen alkion poistettavan paikalle
     * ja määrittää viimeisen alkion tyhjäksi. Päivittää sanojen lukumäärän. 
     * @param sana poistettava sana
     */
    public void poistaSana(String sana) {
        int i = haeIndeksi(sana);
        if (i > -1) poistaSana(i);        
    }
    
    
    /**
     * Jos annetussa indeksissä on sana, poistetaan se. Siirtää taulukon viimeisen alkion poistettavan paikalle
     * ja määrittää viimeisen alkion tyhjäksi. Päivittää sanojen lukumäärän. 
     * @param indeksi indeksi, josta poistetaan.
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat(6, "Yksi Kaksi Kolme Neljä");
     *  a.poistaSana(0);
     *  a.get(0) === "Neljä";
     * </pre>
     */
    public void poistaSana(int indeksi) {
        if (indeksi < -1 || indeksi > sanoja-1) return;
        this.sanat[indeksi] = this.sanat[this.sanoja-1];
        this.sanat[this.sanoja-1] = null;
        this.sanoja--;
    }
    
    
    /**
     * Hakee, mikä on sanan indeksi
     * @param sana haettava sana
     * @return Sanan indeksin. Jos ei löydy, -1.
     */
    public int haeIndeksi(String sana) {        
        for (int i = 0; i < this.sanoja; i++)
            if (sana.equals(this.sanat[i])) return i;
        return -1;
    }
    
    public String[] getSanat() {
        String[] uusi = new String[this.sanoja];
        for (int i = 0; i < sanoja; i++) uusi[i] = this.sanat[i];
        return uusi;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (String s : this.sanat) sb.append(" " + s);
        return sb.toString();
    }
}
